//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//	ChatSelector.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "ChatSelector.h"
#include "packets.h"
#include "otherfunctions.h"
#include "Smtp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif


CChatItem::CChatItem()
{
	log = 0;
	messagepending.Empty();
	notify = false;
	history_pos = 0;
}

//	CChatSelector

IMPLEMENT_DYNAMIC(CChatSelector, CClosableTabCtrl)
CChatSelector::CChatSelector()
{
	m_pCloseBtn = NULL;
	m_pMessageBox = NULL;
	m_pSendBtn = NULL;
	lastemptyicon = false;
	blinkstate = false;
	m_Timer = 0;
}

CChatSelector::~CChatSelector()
{
}


BEGIN_MESSAGE_MAP(CChatSelector, CClosableTabCtrl)
	ON_WM_TIMER()
	ON_NOTIFY_REFLECT(TCN_SELCHANGE, OnTcnSelchangeChatsel)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_CCLOSE, OnBnClickedCclose)
	ON_BN_CLICKED(IDC_CSEND, OnBnClickedCsend)
	ON_NOTIFY(EN_LINK, 124, OnEnLinkMessageBox)
	ON_WM_DESTROY()
END_MESSAGE_MAP()


//	CChatSelector message handlers
void CChatSelector::Init()
{
	CRect rect;
	GetClientRect(&rect);
	AdjustRect(FALSE, rect);

	CRect rClose;
	m_pCloseBtn = GetParent()->GetDlgItem(IDC_CCLOSE);
	m_pCloseBtn->SetParent(this);
	m_pCloseBtn->GetWindowRect(&rClose);
	m_pCloseBtn->SetWindowPos( NULL, rect.right - 7 - rClose.Width(), rect.bottom - 7 - rClose.Height(),
	                           rClose.Width(), rClose.Height(), SWP_NOZORDER );
	CRect rSend;
	m_pSendBtn = GetParent()->GetDlgItem(IDC_CSEND);
	m_pSendBtn->SetParent(this);
	m_pSendBtn->GetWindowRect(&rSend);
	m_pSendBtn->SetWindowPos( NULL, rect.right - 7 - rClose.Width() - 7 - rSend.Width(), rect.bottom - 7 - rSend.Height(),
	                          rSend.Width(), rSend.Height(), SWP_NOZORDER );

	CRect rMessage;
	m_pMessageBox = GetParent()->GetDlgItem(IDC_CMESSAGE);
	m_pMessageBox->SetParent(this);
	m_pMessageBox->GetWindowRect(&rMessage);
	m_pMessageBox->SetWindowPos( NULL, rect.left + 7, rect.bottom - 9 - rMessage.Height(),
	                             rect.right - 7 - rClose.Width() - 7 - rSend.Width() - 21,
	                             rMessage.Height(), SWP_NOZORDER );

	int iTop = rClose.Height() > rSend.Height() ? rClose.Height() : rSend.Height();
	if (iTop < rMessage.Height())
		iTop = rMessage.Height();

	CRect rChatOut = rect;
	rChatOut.top += 7;
	rChatOut.left += 7;
	rChatOut.right -= 18;
	rChatOut.bottom -= 7 - iTop - 7;

	ModifyStyle(0, WS_CLIPCHILDREN);
	chatout.Create(WS_VISIBLE | WS_CHILD | WS_HSCROLL | WS_VSCROLL | ES_MULTILINE | ES_READONLY | ES_NOHIDESEL, rChatOut, this, 124);
	chatout.ModifyStyleEx(0, WS_EX_STATICEDGE, SWP_FRAMECHANGED);
	chatout.AppendHyperLink(_T(""),_T(""),_T("eMule Plus"),_T(""));
	chatout.AppendText( CString(" Version ") + CString(CURRENT_VERSION_LONG) + CString(" - ") + GetResString(IDS_CHAT_WELCOME) );
	chatout.SetEventMask(chatout.GetEventMask() | ENM_LINK);
	chatout.SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
	chatout.SetTitle(GetResString(IDS_CW_MESSAGES));

	m_imageList.Create(16, 16, ILC_COLOR32, 0, 10);
	m_imageList.SetBkColor(RGB(255, 0, 255));
	m_imageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_CHAT), 16, 16));
	m_imageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_MESSAGE), 16, 16));
	m_imageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_MPENDING), 16, 16));
	LoadImgLstIcon(NULL, 0, 0);
	CImageList imageList;
	CBitmap Bitmap, *pOldBitmap;
	CDC *pCtrlDC = GetDC(), TempDC;
	TempDC.CreateCompatibleDC(pCtrlDC);
	Bitmap.CreateCompatibleBitmap(pCtrlDC, 16 * 3, 16);
	pOldBitmap = TempDC.SelectObject(&Bitmap);
	m_imageList.Draw(&TempDC, 0, CPoint(0, 0), ILD_NORMAL);
	m_imageList.Draw(&TempDC, 1, CPoint(16, 0), ILD_NORMAL);
	m_imageList.Draw(&TempDC, 2, CPoint(32, 0), ILD_NORMAL);
	TempDC.SelectObject(pOldBitmap);
	imageList.Create(16, 16, ILC_COLOR32 | ILC_MASK, 0, 10);
	imageList.Add(&Bitmap, RGB(255, 0, 255));
	SetImageList(&imageList);
	imageList.Detach();
	Bitmap.DeleteObject();
	ReleaseDC(pCtrlDC);

	VERIFY((m_Timer = SetTimer(20, 1500, 0)));
}

CChatItem* CChatSelector::StartSession(CUpDownClient* client, bool show)
{
	if (GetTabByClient(client) != 0xFFFF)
	{
		if (show)
		{
			SetCurSel(GetTabByClient(client));
			chatout.SetHyperText(GetItemByClient(client)->log);
			chatout.SetTitle(client->GetUserName());
			if ((chatout.m_bAutoScroll) /*&& (GetItemState(GetTabByClient(client), TCIS_HIGHLIGHTED) & TCIS_HIGHLIGHTED)*/)
				chatout.ScrollToLastLine();
		}
		return 0;
	}

	CChatItem* chatitem = new CChatItem();
	chatitem->client = client;
	chatitem->log = new CPreparedRTFText();

	COleDateTime timelog(COleDateTime::GetCurrentTime());
	CString sessions = GetResString(IDS_CHAT_START) + CString(client->GetUserName()) + _T(" (IP: ") + CString(client->GetFullIP()) + _T(") - ") + timelog.Format() + _T("\n");
	chatitem->log->AppendText(sessions, RGB(255, 0, 0));
	client->SetChatState(MS_CHATTING);

	CString name;
	if (!client->GetUserName().IsEmpty())
	{
		name = client->GetUserName();

		if (name.GetLength() > 30)
			name = name.Left(30) + _T("...");
	}
	else
		name.Format(_T("(%s)"), GetResString(IDS_UNKNOWN));

	TCITEM newitem;
	newitem.mask = TCIF_PARAM | TCIF_TEXT | TCIF_IMAGE;
	newitem.lParam = (LPARAM)chatitem;
	newitem.pszText = name.GetBuffer();
	newitem.cchTextMax = name.GetLength() + 1;
	newitem.iImage = 0;
	uint16 itemnr = InsertItem(GetItemCount(), &newitem);
	if (show)
	{
		SetCurSel(itemnr);
		chatout.SetHyperText(chatitem->log);
		chatout.SetTitle(client->GetUserName());
		if ((chatout.m_bAutoScroll) /*&& (GetItemState(itemnr, TCIS_HIGHLIGHTED) & TCIS_HIGHLIGHTED)*/)
			chatout.ScrollToLastLine();
	}

	if (show || g_eMuleApp.m_pdlgEmule->m_pdlgActive == &g_eMuleApp.m_pdlgEmule->m_wndChat)
		m_pMessageBox->SetFocus();

	return chatitem;
}

uint16 CChatSelector::GetTabByClient(CUpDownClient* client)
{
	for (int i = 0; i < GetItemCount();i++)
	{
		TCITEM cur_item;
		cur_item.mask = TCIF_PARAM;
		GetItem(i, &cur_item);
		if (((CChatItem*)cur_item.lParam)->client == client)
			return i;
	}
	return -1;
}

CChatItem* CChatSelector::GetItemByClient(CUpDownClient* client)
{
	for (int i = 0; i < GetItemCount();i++)
	{
		TCITEM cur_item;
		cur_item.mask = TCIF_PARAM;
		GetItem(i, &cur_item);
		if (((CChatItem*)cur_item.lParam)->client == client)
			return(CChatItem*)cur_item.lParam;
	}
	return 0;
}

void CChatSelector::ProcessMessage(CUpDownClient* sender, LPCTSTR message)
{
	//filter me?
	CString Cmessage = CString(message).MakeLower();
	CString resToken;
	int curPos = 0;

	if ((Cmessage.Find(_T("emule sans upload @ http://www.chez.com/theworld/")) > -1) || (Cmessage.Find(_T("http://fullspeed.to/mison")) > -1) && (g_eMuleApp.m_pGlobPrefs->GetMlDonkeyCM()))
	{
		AddDebugLogLine(false, RGB_LOG_DIMMED + _T("Anti-leechermods: Client %s (%s) has been banned because of spamming"), sender->GetUserName(), GetClientNameAndVersionString(sender));
		sender->Ban(_T("AUTOMATED MESSAGE: Your client is being banned for spamming and leeching!"), GetResString(IDS_BAN_LEECHERSPAM));
		return;
	}

	resToken = g_eMuleApp.m_pGlobPrefs->GetMessageFilter().Tokenize(_T("|"), curPos);
	while (resToken != "")
	{
		if (Cmessage.Find(resToken.MakeLower()) > -1)
		{
			AddDebugLogLine(false, RGB_LOG_DIMMED + _T("Filtered message %s from user %s (%s)"), message, sender->GetUserName(), GetClientNameAndVersionString(sender));
			return;
		}
		resToken = g_eMuleApp.m_pGlobPrefs->GetMessageFilter().Tokenize(_T("|"), curPos);
	};

//	continue
	CChatItem* ci = GetItemByClient(sender);
	bool isNewChatWindow = false;
	if (!ci)
	{
		if (GetItemCount() >= g_eMuleApp.m_pGlobPrefs->GetMsgSessionsMax())
			return;

		if (g_eMuleApp.m_pGlobPrefs->GetAcceptMessagesFrom() == 4) //no messages
			return;

		if (g_eMuleApp.m_pGlobPrefs->GetAcceptMessagesFrom() == 2) //only friends
			if (!(sender->IsFriend()))
				return;

		if ((g_eMuleApp.m_pGlobPrefs->GetAcceptMessagesFrom() == 3) && (!(sender->IsFriend())))
		{ //only friends
			AddLogLine(false, GetResString(IDS_IM_MSGFROMCHAT) + _T(" %s: %s"), sender->GetUserName(), message);
			sender->SetChatState(MS_NONE);
			return;
		}

		ci = StartSession(sender, false);
		if (!ci)
			return;
		isNewChatWindow = true;
	}
	ci->log->AppendText(sender->GetUserName(), RGB(50, 200, 250));

	COleDateTime timelog(COleDateTime::GetCurrentTime());
	CString curtime;
	curtime.Format(_T(" (%s)"), timelog.Format());
	ci->log->AppendText(curtime + _T(": "), RGB(50, 200, 250));
	ci->log->AppendText(CString(message) + _T("\n"));

	if (g_eMuleApp.m_pGlobPrefs->GetAwayState() == true)
	{
		if ((::GetTickCount() - ci->client->GetAwayMessageResendCount()) > 3000)
		{ //send again only if 3 secs from last away message
			SendAwayMessage(ci);
			ci->client->SetAwayMessageResendCount(::GetTickCount());
		}
	}
	if (GetCurSel() != GetTabByClient(sender))
		SetItemState(GetTabByClient(sender), TCIS_HIGHLIGHTED, TCIS_HIGHLIGHTED);
	if (GetCurSel() == GetTabByClient(sender) && GetParent()->IsWindowVisible())
		chatout.SetHyperText(ci->log);
	else if (g_eMuleApp.m_pGlobPrefs->GetAwayState() == false)
	{
		ci->notify = true;
		if (isNewChatWindow || g_eMuleApp.m_pGlobPrefs->GetNotifierPopsEveryChatMsg())
		{
			CString MessageText;
			MessageText.Format(GetResString(IDS_TBN_NEWCHATMSG) + CString(" ") + CString(sender->GetUserName()) + CString(":'") + CString(message) + CString("'\n"));
			g_eMuleApp.m_pSMTPConnection->SendMail(MessageText, g_eMuleApp.m_pGlobPrefs->GetNotifierPopsEveryChatMsg(), g_eMuleApp.m_pGlobPrefs->IsSMTPInfoEnabled());
			g_eMuleApp.m_pdlgEmule->ShowNotifier(MessageText, TBN_CHAT, false, g_eMuleApp.m_pGlobPrefs->GetNotifierPopsEveryChatMsg());
		}
		isNewChatWindow = false;
	}
}

bool CChatSelector::SendMessage(const CString& message)
{
	CChatItem * ci = GetCurrentChatItem();
	if (ci == NULL)
		return false;

	if (ci->history.GetCount() == g_eMuleApp.m_pGlobPrefs->GetMaxChatHistoryLines())
		ci->history.RemoveAt(0);
	ci->history.Add(CString(message));
	ci->history_pos = ci->history.GetCount();

	if (ci->client->GetChatState() == MS_CONNECTING)
		return false;
#ifdef OLD_SOCKETS_ENABLED
	if (ci->client->m_pRequestSocket && ci->client->m_pRequestSocket->IsConnected())
	{
		SendMessagePacket(*ci, message);
		ci->log->AppendText(g_eMuleApp.m_pGlobPrefs->GetUserNick(), RGB(1, 180, 20));
		COleDateTime timelog(COleDateTime::GetCurrentTime());
		CString curtime;
		curtime.Format(_T(" (%s)"), timelog.Format());
		ci->log->AppendText(curtime + _T(": "), RGB(1, 180, 20));
		ci->log->AppendText(message + _T("\n"));
	}
	else
	{
		ci->log->AppendText(_T("*** ") + GetResString(IDS_CONNECTING), RGB(255, 0, 0));
		ci->messagepending = message;
		ci->client->SetChatState(MS_CONNECTING);
		ci->client->TryToConnect();
	}
#endif //OLD_SOCKETS_ENABLED
	if (chatout.GetHyperText() == ci->log)
		chatout.UpdateSize();
	return true;
}

void CChatSelector::ConnectingResult(CUpDownClient* sender, bool success)
{
	CChatItem * ci = GetItemByClient(sender);
	if (!ci)
		return;
	ci->client->SetChatState(MS_CHATTING);
	if (!success)
	{
		if (!ci->messagepending.IsEmpty())
		{
			ci->log->AppendText(_T(" ") + GetResString(IDS_FAILED) + _T("\n"), RGB(255, 0, 0));
			ci->messagepending.Empty();	//SyruS i wish compiler would warn on this: delete[] ci->messagepending;
		}
		else
			ci->log->AppendText(GetResString(IDS_CHATDISCONNECTED) + _T("\n"), RGB(255, 0, 0));
	}
	else
	{
		ci->log->AppendText(_T(" ok\n"), RGB(255, 0, 0));
		SendMessagePacket(*ci, ci->messagepending);
		ci->log->AppendText(g_eMuleApp.m_pGlobPrefs->GetUserNick() + _T(": "), RGB(1, 180, 20));
		ci->log->AppendText(ci->messagepending + _T("\n"));
		ci->messagepending.Empty();
	}
	if (chatout.GetHyperText() == ci->log)
		chatout.UpdateSize();
}

void CChatSelector::DeleteAllItems()
{
	for (int i = 0; i < GetItemCount();i++)
	{
		TCITEM cur_item;
		cur_item.mask = TCIF_PARAM;
		GetItem(i, &cur_item);
		delete (CChatItem*)cur_item.lParam;
	}
}

void CChatSelector::OnTimer(UINT_PTR nIDEvent)
{
	blinkstate = !blinkstate;
	bool globalnotify = false;
	for (int i = 0; i < GetItemCount();i++)
	{
		TCITEM cur_item;
		cur_item.mask = TCIF_PARAM;
		GetItem(i, &cur_item);
		cur_item.mask = TCIF_IMAGE;
		if (((CChatItem*)cur_item.lParam)->notify)
		{
			cur_item.iImage = (blinkstate) ? 1 : 2;
			SetItem(i, &cur_item);
			globalnotify = true;
		}
		else if (cur_item.iImage != 0)
		{
			cur_item.iImage = 0;
			SetItem(i, &cur_item);
		}
	}
	if (globalnotify)
	{
		g_eMuleApp.m_pdlgEmule->ShowMessageState(((blinkstate) ? 1 : 2));
		lastemptyicon = false;
	}
	else if (!lastemptyicon)
	{
		g_eMuleApp.m_pdlgEmule->ShowMessageState(0);
		lastemptyicon = true;
	}
}

CChatItem* CChatSelector::GetCurrentChatItem()
{
	if (GetCurSel() == ( -1))
		return NULL;
	TCITEM cur_item;
	cur_item.mask = TCIF_PARAM;
	GetItem(GetCurSel(), &cur_item);
	CChatItem* ci = (CChatItem*)cur_item.lParam;
	return ci;
}

void CChatSelector::ShowChat()
{
	if (GetCurSel() == ( -1))
		return;
	/*
	TCITEM cur_item;
	cur_item.mask = TCIF_PARAM;
	GetItem(GetCurSel(),&cur_item);*/

	CChatItem* ci = GetCurrentChatItem();
	chatout.SetHyperText(ci->log);
	chatout.SetTitle(ci->client->GetUserName());
	ci->notify = false;
	m_pMessageBox->SetFocus();
	if ((chatout.m_bAutoScroll) /*&& (GetItemState(GetCurSel(), TCIS_HIGHLIGHTED) & TCIS_HIGHLIGHTED)*/)
		chatout.ScrollToLastLine();
	SetItemState(GetCurSel(), TCIS_HIGHLIGHTED, NULL);
}

void CChatSelector::OnTcnSelchangeChatsel(NMHDR *pNMHDR, LRESULT *pResult)
{
	ShowChat();
	*pResult = 0;
}

INT	CChatSelector::InsertItem(int nItem, TCITEM* pTabCtrlItem)
{
	if (!GetItemCount())
	{
		WINDOWPLACEMENT wp;
		chatout.GetWindowPlacement(&wp);
		wp.rcNormalPosition.top += 20;
		chatout.SetWindowPlacement(&wp);
	}
	int result = CClosableTabCtrl::InsertItem(nItem, pTabCtrlItem);
	RedrawWindow();
	return result;
}

BOOL CChatSelector::DeleteItem(int nItem)
{
	CClosableTabCtrl::DeleteItem(nItem);
	if (!GetItemCount())
	{
		WINDOWPLACEMENT wp;
		chatout.GetWindowPlacement(&wp);
		wp.rcNormalPosition.top -= 20;
		chatout.SetWindowPlacement(&wp);
		chatout.Reset();
		chatout.AppendHyperLink(_T(""),_T(""),_T("eMule Plus"),_T(""));
		chatout.AppendText( CString(" Version ") + CString(CURRENT_VERSION_LONG) + CString(" - ") + GetResString(IDS_CHAT_WELCOME) );
		chatout.SetTitle(GetResString(IDS_CW_MESSAGES));
	}
	RedrawWindow();
	return true;
}

void CChatSelector::EndSession(CUpDownClient* client)
{
	sint16 usedtab;
	if (client)
		usedtab = GetTabByClient(client);
	else
		usedtab = GetCurSel();
	if (usedtab == ( -1))
		return;
	TCITEM item;
	item.mask = TCIF_PARAM;
	GetItem(usedtab, &item);
	CChatItem* ci = (CChatItem*)item.lParam;
	ci->client->SetChatState(MS_NONE);

	DeleteItem(usedtab);
	if (chatout.GetHyperText() == ci->log)
		chatout.SetHyperText(0);
	delete ci;
}

void CChatSelector::OnSize(UINT nType, int cx, int cy)
{
	CClosableTabCtrl::OnSize(nType, cx, cy);

	CRect rect;
	GetClientRect(&rect);
	AdjustRect(FALSE, rect);

	CRect rClose;
	m_pCloseBtn->GetWindowRect(&rClose);
	m_pCloseBtn->SetWindowPos( NULL, rect.right - 7 - rClose.Width(), rect.bottom - 7 - rClose.Height(),
	                           rClose.Width(), rClose.Height(), SWP_NOZORDER );
	CRect rSend;
	m_pSendBtn->GetWindowRect(&rSend);
	m_pSendBtn->SetWindowPos( NULL, rect.right - 7 - rClose.Width() - 7 - rSend.Width(), rect.bottom - 7 - rSend.Height(),
	                          rSend.Width(), rSend.Height(), SWP_NOZORDER );

	CRect rMessage;
	m_pMessageBox->GetWindowRect(&rMessage);
	m_pMessageBox->SetWindowPos( NULL, rect.left + 7, rect.bottom - 9 - rMessage.Height(),
	                             rect.right - 7 - rClose.Width() - 7 - rSend.Width() - 21,
	                             rMessage.Height(), SWP_NOZORDER );

	int iTop = rClose.Height() > rSend.Height() ? rClose.Height() : rSend.Height();
	if (iTop < rMessage.Height())
		iTop = rMessage.Height();

	chatout.SetWindowPos(NULL, rect.left + 7, rect.top + 7, rect.right - 18, rect.Height() - 7 - iTop - 14, SWP_NOZORDER);
}

void CChatSelector::OnBnClickedCclose()
{
	EndSession();

	int iTotalTabs = GetItemCount();

	if(iTotalTabs != -1 && iTotalTabs > 0)
	{
		SetCurSel(iTotalTabs-1);
		ShowChat();
	}
	else if (iTotalTabs != -1 && iTotalTabs == 0)
	{
		SetCurSel(iTotalTabs);
		ShowChat();
	}
}

void CChatSelector::OnBnClickedCsend()
{
	uint16 len = m_pMessageBox->GetWindowTextLength() + 2;
	TCHAR* messagetosend = new TCHAR[len + 1];
	m_pMessageBox->GetWindowText(messagetosend, len);
	if (SendMessage(messagetosend))
		m_pMessageBox->SetWindowText(_T(""));
	delete[] messagetosend;
}

BOOL CChatSelector::PreTranslateMessage(MSG* pMsg)
{
	if (pMsg->message == WM_KEYDOWN)
	{

		if (pMsg->wParam == VK_RETURN)
		{
			if (pMsg->hwnd == m_pMessageBox->m_hWnd)
				OnBnClickedCsend();
		}

		if ( (pMsg->hwnd == GetDlgItem(IDC_CMESSAGE)->m_hWnd) &&
		     (pMsg->wParam == VK_UP || pMsg->wParam == VK_DOWN) )
		{
			g_eMuleApp.m_pdlgEmule->m_wndChat.ScrollHistory(pMsg->wParam == VK_DOWN);
			return TRUE;
		}
	}
	return CClosableTabCtrl::PreTranslateMessage(pMsg);
}

void CChatSelector::Localize(void)
{
	if (m_hWnd)
	{
		if (m_pSendBtn)
			m_pSendBtn->SetWindowText(GetResString(IDS_CW_SEND));
		else
			GetParent()->GetDlgItem(IDC_CSEND)->SetWindowText(GetResString(IDS_CW_SEND));
		if (m_pCloseBtn)
			m_pCloseBtn->SetWindowText(GetResString(IDS_CW_CLOSE));
		else
			GetParent()->GetDlgItem(IDC_CCLOSE)->SetWindowText(GetResString(IDS_CW_CLOSE));
	}
}

void CChatSelector::OnEnLinkMessageBox(NMHDR *pNMHDR, LRESULT *pResult)
{
	*pResult = 0;
	ENLINK* pEnLink = reinterpret_cast<ENLINK *>(pNMHDR);
	if (pEnLink && pEnLink->msg == WM_LBUTTONDOWN)
	{
		CString strUrl;
		chatout.GetTextRange(pEnLink->chrg.cpMin, pEnLink->chrg.cpMax, strUrl);
		CString sLangUrl = strUrl; // GetTextRange returns '?' for non-english chars :(
		sLangUrl.Remove('?'); sLangUrl.Remove(' ');
		if (strUrl == _T("eMule Plus") || sLangUrl.IsEmpty())
			ShellExecute(NULL, NULL, _T("http://emuleplus.info"), NULL, NULL, SW_SHOWDEFAULT);
		*pResult = 1;
	}
}

void CChatSelector::OnDestroy()
{
	if (m_Timer)
		KillTimer(m_Timer);
	CClosableTabCtrl::OnDestroy();
}

bool CChatSelector::SendAwayMessage(CChatItem* ci)
{
	if (!ci)
		return false;
	if (ci->client->GetChatState() == MS_CONNECTING)
		return false; //send AwayMessage only if connected
#ifdef OLD_SOCKETS_ENABLED
	if (ci->client->m_pRequestSocket && ci->client->m_pRequestSocket->IsConnected())
	{
		CString awm = CString("[AUTO REPLY:AWAY] ") + CString(g_eMuleApp.m_pGlobPrefs->GetAwayStateMessage());
		uint16 mlen = (uint16)_tcslen(LPCTSTR(awm));
		Packet* packet = new Packet(OP_MESSAGE, mlen + 2);
		memcpy2(packet->m_pcBuffer, &mlen, 2);
		memcpy2(packet->m_pcBuffer + 2, awm, mlen);
		ci->client->m_pRequestSocket->SendPacket(packet, true, true);
		ci->log->AppendText(g_eMuleApp.m_pGlobPrefs->GetUserNick(), RGB(1, 180, 20));
		COleDateTime timelog(COleDateTime::GetCurrentTime());
		CString curtime;
		curtime.Format(_T(" (%s)"), timelog.Format());
		ci->log->AppendText(curtime + _T(": "), RGB(1, 180, 20));
		ci->log->AppendText(CString(awm) + _T("\n"));
	}
#endif //OLD_SOCKETS_ENABLED
	if (chatout.GetHyperText() == ci->log)
		chatout.UpdateSize();
	return true;
}

void CChatSelector::SendMessagePacket(CChatItem& ci, const CString& msg)
{
	uint16 mlen = msg.GetLength();
	Packet* packet = new Packet(OP_MESSAGE, mlen + 2);
	memcpy2(packet->m_pcBuffer, &mlen, 2);
	memcpy2(packet->m_pcBuffer + 2, msg.GetString(), mlen);
	g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(packet->m_dwSize);
#ifdef OLD_SOCKETS_ENABLED

	ci.client->m_pRequestSocket->SendPacket(packet, true, true);
#endif //OLD_SOCKETS_ENABLED
}
