// TaskProcessor_Main.cpp: implementation of the CTaskProcessor_Sockets class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "TaskProcessorSockets.h"
#include "../EmEngine.h"

//////////////////////////////////////////////////////////////////////
// starting
bool CTaskProcessor_Sockets::Start()
{
//	m_mapClients.InitHashTable(997);
	m_dwWaitTimeout = 500;
	return true;
}

//////////////////////////////////////////////////////////////////////
// destroy all clients that are possibly left in our map
void CTaskProcessor_Sockets::Stop()
{
	
	for (SocketClientMap::iterator pos = m_mapClients.begin(); pos != m_mapClients.end(); pos++)
	{
		CEmClient* pClient = (*pos).second;
		ASSERT(pClient);
		delete pClient;
	}
	m_mapClients.clear();
}

//////////////////////////////////////////////////////////////////////
// processing timeout
// this should replace all old Process() functions
void CTaskProcessor_Sockets::ProcessTimeout()
{
	// keep connection alive once per minute
	// recommended by lugdunummaster
	if(m_stEngine.IsConnected())
	{
		// sending empty OFFER FILES opcode
	}

	m_dwWaitTimeout = 1000; // perform check once per second
}

//////////////////////////////////////////////////////////////////////
// creating new client object
CEmClient* CTaskProcessor_Sockets::AllocClient(T_CLIENT_TYPE eType)
{
	CEmClient* pClient;
	switch (eType)
	{
	case T_CLIENT_PEER:
		pClient = new CEmClient_Peer;
		break;
	case T_CLIENT_SERVER:
		pClient = new CEmClient_Server;
		break;
	case T_CLIENT_WEB:
		pClient = new CEmClient_Web;
		break;
	default:
		ASSERT(FALSE); // invalid type ???
		return NULL;
	}
	if (pClient)
		pClient->m_nClientID = ++m_nLastClientID;
	return pClient;
}

//////////////////////////////////////////////////////////////////////
// connecting to any client (or server)
CEmClient* CTaskProcessor_Sockets::AllocTcpConnect(ULONG nAddr, USHORT nPort, T_CLIENT_TYPE eClientType)
{
	CEmClient* pClient = AllocClient(eClientType);
	if (pClient)
	{
		pClient->m_bFromOutside = false;
		pClient->m_nAddr = nAddr;
		pClient->m_uPort = nPort;
		SOCKET hSocket = m_stEngine.m_stTcp.AllocConnect(nAddr, nPort, eClientType);
		if (INVALID_SOCKET == hSocket)
		{
			delete pClient;
			pClient = NULL;
		} else
		{
			pClient->m_hSocket = hSocket;
			pClient->m_bIsConnected = false;

			m_mapClients[hSocket] = pClient;
		}
	} 
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "no memory");
	}

	return pClient;
}

//////////////////////////////////////////////////////////////////////
// disconnect existing client and destroy its object
void CTaskProcessor_Sockets::KillClient(CEmClient* pClient)
{
	ASSERT(pClient);
	SOCKET hSocket = pClient->m_hSocket;
	pClient->OnDisconnected(m_stEngine);
	delete pClient;
	m_mapClients.erase(hSocket);
}
