// TaskProcessor_Main.h: interface for the CTaskProcessor_Sockets class.
//
//////////////////////////////////////////////////////////////////////
#ifdef NEW_SOCKETS
#if !defined(AFX_TASKPROCESSOR_MAIN_H__E3110A38_4DB8_41A3_B7B4_F4481D9DDCB1__INCLUDED_)
#define AFX_TASKPROCESSOR_MAIN_H__E3110A38_4DB8_41A3_B7B4_F4481D9DDCB1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../../Loggable.h"
#include "../TaskProcessor.h"

class CEmEngine;

struct CEmClient : public CLoggable {
	SOCKET	m_hSocket;
	bool	m_bIsConnected;
	bool	m_bFromOutside;

	ULONG	m_nAddr;
	USHORT	m_uPort;
	ULONG	m_nClientID; // must be generated automatically

	virtual ~CEmClient();
	virtual T_CLIENT_TYPE GetType() const = 0;

	virtual void OnConnected(CEmEngine&);
	virtual void OnAccepted(CEmEngine&);
	virtual void OnDisconnected(CEmEngine&);
};

struct CEmClient_Peer : public CEmClient {
	virtual T_CLIENT_TYPE GetType() const;
	virtual void OnConnected(CEmEngine&);
};

struct CEmClient_Server : public CEmClient {
	virtual T_CLIENT_TYPE GetType() const;
	virtual void OnConnected(CEmEngine&);
	virtual void OnDisconnected(CEmEngine&);

	int	m_nCredits; // Will use later to check for credits left
	int	m_nMsgCount; // To check that server version appears in first message
};

struct CEmClient_Web : public CEmClient {

	virtual T_CLIENT_TYPE GetType() const;
	// Possibly some user's stuff.
	virtual void OnConnected(CEmEngine&);
};

typedef map<SOCKET, CEmClient*> SocketClientMap;

class CTaskProcessor_Sockets : public CTaskProcessor
{
	virtual bool Start();
	virtual void Stop();
	virtual void ProcessTimeout();

	ULONG m_nLastClientID;

public:
	CTaskProcessor_Sockets(CEmEngine& stEngine) :
		CTaskProcessor(stEngine),
		m_nLastClientID(0)
		{}

	SocketClientMap m_mapClients;

	CEmClient* AllocClient(T_CLIENT_TYPE);
	CEmClient* CTaskProcessor_Sockets::Lookup(SOCKET hSocket)
	{
		SocketClientMap::iterator pos = m_mapClients.find(hSocket);
		return (pos != m_mapClients.end() ? (*pos).second : NULL);
	}

	CEmClient* AllocTcpConnect(ULONG nAddr, USHORT nPort, T_CLIENT_TYPE);

	void KillClient(CEmClient*);
};
#endif // !defined(AFX_TASKPROCESSOR_MAIN_H__E3110A38_4DB8_41A3_B7B4_F4481D9DDCB1__INCLUDED_)
#endif