//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// handling incoming connections (up or downloadrequests)

#pragma once
#include "preferences.h"
#include "packets.h"
#include "emsocket.h"


class CUpDownClient;
class CPacket;
class CTimerWnd;

class CClientReqSocket : public CEMSocket
{
	friend class CListenSocket;

public:
	CUpDownClient	   *m_pClient;

private:
	CPreferences	   *m_pPrefs;
	uint32				m_dwTimeoutTimer;
	bool				m_bDeleteThis;
	uint32				m_dwDeleteTimer;

public:
						CClientReqSocket(CPreferences *in_pPrefs, CUpDownClient *in_pClient = NULL);
	void				Disconnect();

	void				ResetTimeOutTimer();
	bool				CheckTimeOut();
	void				Safe_Delete();
	bool				IsDeleted() { return m_bDeleteThis; }
	
	bool				Create();

protected:
	virtual void		Close()	{CAsyncSocketEx::Close();} // deadlake PROXYSUPPORT - changed to AsyncSocketEx
	virtual	void		OnInit();
	void				OnClose(int nErrorCode);
	void				OnSend(int nErrorCode);
	void				OnReceive(int nErrorCode);
	void				OnError(int nErrorCode);
	void				PacketReceived(Packet* packet);

private:
	void				TimedDelete();
					   ~CClientReqSocket();

	bool				ProcessPacket(char* pcPacketBuf, uint32 dwSize,EnumOpcodes eOpcode);
	bool				ProcessExtPacket(char* pcPacketBuf, uint32 dwSize,EnumOpcodes eOpcode);
//	void				PacketToDebugLogLine(const char* protocol, const char* packet, uint32 size, uint8 opcode) const;
};

// CListenSocket command target
class CListenSocket : public CAsyncSocketEx // deadlake PROXYSUPPORT - changed to AsyncSocketEx
{
private:
	bool			m_bListening;
	CPreferences   *m_pPrefs;

	CTypedPtrList<CPtrList, CClientReqSocket*>		m_openSocketList;

	uint16			m_iNumOpenSockets;
	uint16			m_iOpenSocketsInterval;
	uint32			m_iMaxConnectionsReachedCount;
	uint16			m_ConnectionStates[3];
	uint16			m_iNumPendingConnections;

public:
					CListenSocket(CPreferences *in_pPrefs);
				   ~CListenSocket();
	bool			StartListening();
	void			StopListening();
	virtual void	OnAccept(int iErrorCode);
	void			Process();
	void			RemoveSocket(CClientReqSocket *pSocket);
	void			AddSocket(CClientReqSocket *pSocket);
	bool			DeleteSocket(CClientReqSocket *pSocket);
	uint16			GetNumOpenSockets()		{return m_openSocketList.GetCount();}
	void			KillAllSockets();
	bool			TooManySockets(bool bIgnoreInterval = false);
	uint32			GetMaxConnectionsReachedCount()	{return m_iMaxConnectionsReachedCount;}
	bool			IsValidSocket(CClientReqSocket *pSocket);
	void			AddConnection();
	void			RecalculateStats();
	void			RestartListening();
	void			Debug_ClientDeleted(CUpDownClient *pClient);
};
