//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"
#include "emuleDlg.h"
#include "knownfilelist.h"
#include "preferences.h"
#ifdef OLD_SOCKETS_ENABLED
#include "sockets.h"
#endif //OLD_SOCKETS_ENABLED
#include "ServerList.h"
#include "SharedFileList.h"
#include "SearchList.h"
#ifdef OLD_SOCKETS_ENABLED
#include "ListenSocket.h"
#endif //OLD_SOCKETS_ENABLED
#include "UploadQueue.h"
#include "DownloadQueue.h"
#include "DownloadList.h"
#include "ClientList.h"
#include "ClientCredits.h"
#include "FriendList.h"
#include "filehashcontrol.h" // Lord KiRon
#ifdef OLD_SOCKETS_ENABLED
#include "clientudpsocket.h"
#endif //OLD_SOCKETS_ENABLED
#include "mdump.h"
#include "IPFilter.h"
#include "Loggable.h"
#include <afxmt.h>
#include "MMServer.h"
#include "smtp.h"
#include "fakecheck.h" //FakeCheck

#include "BerkeleyDb/build_win32/db_cxx.h"
#include <afxcoll.h>

class CSearchList;
class CUploadQueue;
#ifdef OLD_SOCKETS_ENABLED
class CListenSocket;
#endif //OLD_SOCKETS_ENABLED
class CDownloadQueue;
class CWebServer;

class CEmuleApp : public CWinApp, public CLoggable
{
public:
	enum AppState
	{
		APP_STATE_RUNNING = 0,
		APP_STATE_SHUTINGDOWN,
		APP_STATE_DONE
	};

	CEmuleApp();
	CEmuleDlg		   *m_pdlgEmule;
	CDownloadList	   *m_pDownloadList;
	CClientList		   *m_pClientList;
	CKnownFileList	   *m_pKnownFilesList;
	CPreferences	   *m_pGlobPrefs;
#ifdef OLD_SOCKETS_ENABLED
	CServerConnect	   *m_pServerConnect;
#endif //OLD_SOCKETS_ENABLED
	CServerList		   *m_pServerList;	
	CSharedFileList	   *m_pSharedFilesList;
	CSearchList		   *m_pSearchList;
#ifdef OLD_SOCKETS_ENABLED
	CListenSocket	   *m_pListenSocket;
#endif //OLD_SOCKETS_ENABLED
	CUploadQueue	   *m_pUploadQueue;
	CDownloadQueue	   *m_pDownloadQueue;
	CClientCreditsList *m_pClientCreditList;
	CFriendList		   *m_pFriendList;
#ifdef OLD_SOCKETS_ENABLED
	CClientUDPSocket   *m_pClientUDPSocket;
#endif //OLD_SOCKETS_ENABLED
	CWebServer		   *m_pWebServer; // kuchin

	virtual BOOL		InitInstance();

	CString			   *m_pstrPendingLink;
	tagCOPYDATASTRUCT  	m_sendStruct; // Cax2
	CFileHashControl 	m_fileHashControl; // Lord KiRon
	CMutex				m_mutexHashing;
	CIPFilter		   *m_pIPFilter;
	CFakecheck		   *m_pFakeCheck; //MORPH

#ifdef MOBILE_MULE
	CMMServer		   *m_pMMServer;
#endif //MOBILE_MULE
	CSMTPConnection	   *m_pSMTPConnection;

	uint64				stat_sessionReceivedBytes;
	uint64				stat_sessionSentBytes;
	uint16				stat_reconnects;
	DWORD				stat_transferStarttime;
	DWORD				stat_serverConnectTime;
	DWORD				stat_starttime;
	uint16				stat_filteredclients;
	ULONGLONG			m_qwComCtrlVer;

	DbEnv			   *pDbEnv;	// Database environment
	Db				   *pDbSLS;	// Stored sources

	CArray<CString,CString> m_strWebServiceURLArray;

	// DonGato: ICC customization
	void				InitURLs();
	CStringArray		m_ICCURLs;

// Implementierung
	// ed2k link functions
	CString		StripInvalidFilenameChars(CString strText, bool bKeepSpaces = true);	
	bool		CopyTextToClipboard(const CString& strText) const;
	CString		CopyTextFromClipboard(); //<<-- enkeyDEV(Ottavio84) -Download all ed2ks in clipboard-
	void		OnlineSig(); 
	void		UpdateReceivedBytes(int32 bytesToAdd);
	void		UpdateSentBytes(int32 bytesToAdd);

	DECLARE_MESSAGE_MAP()
protected:
	int ProcessCommandline();
	void SetTimeOnTransfer();
	static BOOL CALLBACK SearchEmuleWindow(HWND hWnd, LPARAM lParam);
	void OnHelp();

	HIMAGELIST			m_hSystemImageList;
	CMapStringToPtr		m_aExtToSysImgIdx;
	CSize				m_sizSmallSystemIcon;

public:
	AppState	m_app_state; // defines application state for shutdown 
	virtual int ExitInstance();

	int			GetFileTypeSystemImageIdx(LPCTSTR pszFilePath, int iLength = -1);
	HIMAGELIST	GetSystemImageList() { return m_hSystemImageList; }
	CSize		GetSmallSytemIconSize() { return m_sizSmallSystemIcon; }

private:
	MiniDumper *m_pDump;
	HANDLE m_hMutexOneInstance;
public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
};
extern CEmuleApp g_eMuleApp;